#-----------------------------------------------------------------------
# File    : dtree.mak
# Contents: commands to build decision and regression tree programs
# Author  : Christian Borgelt
# History : 27.01.2003 file created
#-----------------------------------------------------------------------
CC       = cl.exe
LD       = link.exe
DEFS     = /D "WIN32" /D "NDEBUG" /D "_CONSOLE" /D "_MBCS"
CFLAGS   = /nologo /ML /W3 /GX /O2 $(DEFS) /YX /FD /c
LDFLAGS  = /nologo /subsystem:console /incremental:no /machine:I386
LIBS     = kernel32.lib user32.lib gdi32.lib winspool.lib comdlg32.lib \
           advapi32.lib shell32.lib ole32.lib oleaut32.lib uuid.lib \
           odbc32.lib odbccp32.lib
INC      = /I $(UTILDIR) /I $(MATHDIR) /I $(TABLEDIR)

THISDIR  = ..\..\dtree\src
UTILDIR  = ..\..\util\src
MATHDIR  = ..\..\math\src
TABLEDIR = ..\..\table\src
HDRS     = $(UTILDIR)\tfscan.h     $(UTILDIR)\vecops.h \
           $(UTILDIR)\scan.h       $(TABLEDIR)\attset.h \
           $(TABLEDIR)\table.h     $(TABLEDIR)\io.h dtree.h
OBJS     = $(UTILDIR)\tfscan.obj   $(UTILDIR)\vecops.obj \
           $(UTILDIR)\scan.obj     $(TABLEDIR)\attset1.obj \
           $(TABLEDIR)\attset2.obj $(TABLEDIR)\attset3.obj
DTI_O    = $(OBJS) $(TABLEDIR)\table1.obj $(TABLEDIR)\io_tab.obj \
           $(MATHDIR)\gamma.obj ft_eval.obj vt_eval.obj \
           dtree.obj dt_grow.obj dti.obj
DTP_O    = $(OBJS) $(TABLEDIR)\table1.obj $(TABLEDIR)\io_tab.obj \
           frqtab.obj vartab.obj dt_exec.obj dt_prune.obj dtp.obj
DTX_O    = $(OBJS) $(TABLEDIR)\io.obj dt_exec.obj dtx.obj
DTR_O    = $(OBJS) rules.obj dt_rule.obj dtr.obj
RSX_O    = $(OBJS) $(TABLEDIR)\io.obj rs_parse.obj rsx.obj
PRGS     = dti.exe dtp.exe dtx.exe dtr.exe rsx.exe

#-----------------------------------------------------------------------
# Build Programs
#-----------------------------------------------------------------------
all:          $(PRGS)

dti.exe:      $(DTI_O) dtree.mak
	$(LD) $(LDFLAGS) $(DTI_O) $(LIBS) /out:$@

dtp.exe:      $(DTP_O) dtree.mak
	$(LD) $(LDFLAGS) $(DTP_O) $(LIBS) /out:$@

dtx.exe:      $(DTX_O) dtree.mak
	$(LD) $(LDFLAGS) $(DTX_O) $(LIBS) /out:$@

dtr.exe:      $(DTR_O) dtree.mak
	$(LD) $(LDFLAGS) $(DTR_O) $(LIBS) /out:$@

rsx.exe:      $(RSX_O) dtree.mak
	$(LD) $(LDFLAGS) $(RSX_O) $(LIBS) /out:$@

#-----------------------------------------------------------------------
# Main Programs
#-----------------------------------------------------------------------
dti.obj:      $(HDRS) dti.c dtree.mak
	$(CC) $(CFLAGS) $(INC) dti.c /Fo$@

dtp.obj:      $(HDRS) dtp.c dtree.mak
	$(CC) $(CFLAGS) $(INC) dtp.c /Fo$@

dtx.obj:      $(HDRS) dtx.c dtree.mak
	$(CC) $(CFLAGS) $(INC) dtx.c /Fo$@

dtr.obj:      $(HDRS) rules.h dtr.c dtree.mak
	$(CC) $(CFLAGS) $(INC) dtr.c /Fo$@

rsx.obj:      $(HDRS) rules.h rsx.c dtree.mak
	$(CC) $(CFLAGS) $(INC) rsx.c /Fo$@

#-----------------------------------------------------------------------
# Frequency Table Management
#-----------------------------------------------------------------------
frqtab.obj:   frqtab.h frqtab.c dtree.mak
	$(CC) $(CFLAGS) $(INC) -c frqtab.c /Fo$@

ft_eval.obj:  frqtab.h frqtab.c dtree.mak
	$(CC) $(CFLAGS) $(INC) /D FT_EVAL frqtab.c /Fo$@

#-----------------------------------------------------------------------
# Variation Table Management
#-----------------------------------------------------------------------
vartab.obj:   vartab.h vartab.c dtree.mak
	$(CC) $(CFLAGS) vartab.c /Fo$@

vt_eval.obj:  vartab.h vartab.c dtree.mak
	$(CC) $(CFLAGS) /D VT_EVAL vartab.c /Fo$@

#-----------------------------------------------------------------------
# Decision and Regression Tree Management
#-----------------------------------------------------------------------
dtree.obj:    $(HDRS) dtree1.c dtree.mak
	$(CC) $(CFLAGS) $(INC) dtree1.c /Fo$@

dt_grow.obj:  $(HDRS) dtree2.c dtree.mak
	$(CC) $(CFLAGS) $(INC) /D DT_GROW dtree2.c /Fo$@

dt_prune.obj: $(HDRS) dtree2.c dtree.mak
	$(CC) $(CFLAGS) $(INC) /D DT_PRUNE dtree2.c /Fo$@

dt_exec.obj:  $(HDRS) dtree1.c dtree.mak
	$(CC) $(CFLAGS) $(INC) /D DT_PARSE dtree1.c /Fo$@

dt_rule.obj:  $(HDRS) dtree1.c dtree.mak
	$(CC) $(CFLAGS) $(INC) /D DT_PARSE /D DT_RULES dtree1.c /Fo$@

#-----------------------------------------------------------------------
# Rule and Rule Set Management
#-----------------------------------------------------------------------
rules.obj:    rules.h rules.c dtree.mak
	$(CC) $(CFLAGS) $(INC) /D RS_DESC rules.c /Fo$@

rs_parse.obj: rules.h rules.c dtree.mak
	$(CC) $(CFLAGS) $(INC) /D RS_DESC /D RS_PARSE rules.c /Fo$@

#-----------------------------------------------------------------------
# External Modules
#-----------------------------------------------------------------------
$(UTILDIR)\vecops.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak vecops.obj
	cd $(THISDIR)
$(UTILDIR)\tfscan.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak tfscan.obj
	cd $(THISDIR)
$(UTILDIR)\scan.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak scan.obj
	cd $(THISDIR)
$(MATHDIR)\gamma.obj:
	cd $(MATHDIR)
	$(MAKE) /f math.mak gamma.obj
	cd $(THISDIR)
$(TABLEDIR)\attset1.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak attset1.obj
	cd $(THISDIR)
$(TABLEDIR)\attset2.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak attset2.obj
	cd $(THISDIR)
$(TABLEDIR)\attset3.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak attset3.obj
	cd $(THISDIR)
$(TABLEDIR)\table1.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak table1.obj
	cd $(THISDIR)
$(TABLEDIR)\io.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak io.obj
	cd $(THISDIR)
$(TABLEDIR)\io_tab.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak io_tab.obj
	cd $(THISDIR)

#-----------------------------------------------------------------------
# Install
#-----------------------------------------------------------------------
install:
	-@copy *.exe c:\home\bin

#-----------------------------------------------------------------------
# Clean up
#-----------------------------------------------------------------------
clean:
	$(MAKE) /f dtree.mak localclean
	cd $(UTILDIR)
	$(MAKE) /f util.mak clean
	cd $(MATHDIR)
	$(MAKE) /f math.mak clean
	cd $(TABLEDIR)
	$(MAKE) /f table.mak localclean
	cd $(THISDIR)

localclean:
	-@erase /Q *~ *.obj *.idb *.pch $(PRGS)
