#! /usr/bin/env python
#
# Gnome Data Miner: GNOME interface to Borgelt's bayes
#
# Copyright (c) Graham J Williams
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. See the file gpl-license.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# TODO
#
#	Experiment with the new logging package
#
#	Complete File Options
#
#	Open => prompt for a project to load
#	Save => Collect set up and save as a project
#	Save As => Collect set up and save as a project
#	Edit => Remove copy, cut, paste clear.
#	Edit -> Properties => Location of programs
#	Edit -> Preferences => ?
#
#	Introduce concept of project: saves file names and all options.
#
#	How to add tooltips to Measure items?
#
#	Why don't the tooltips on the toolbar work?
#
#	There are too many literals throughout.  Name the constants (like
#	default classificaiton measure) and use them instead.
#
#	Plenty of opportunity to tidy up the code.
#
# Style Guide (see http://www.python.org/doc/essays/styleguide.html)
#
#    Use the "_" convention for function/variable naming (e.g., main_window).
#	Consistent with glade naming. No conflict with Class/Module names.
#    Use Python doc strings to comment each method (triple quotes).
#       Refer to http://www.python.org/peps/pep-0257.html
#    Document functions in docstrings, not in comments, so they are available.
#	Automated tools can extract the docstrings, but not the comments.
#    Use same names in Python as for the Glade objects.
#	No need to remember two names for everything.
#    Classes and Modules generally use the CapWords naming style.
#
# Authors: Graham Williams
#

########################################################################
#
# Libraries
#
import os
import sys
import popen2
import tempfile
try:
    import gtk
    import gtk.glade
except ImportError, e:
    print e
    print "Please install python-glade2 and python-gnome2."
    sys.exit(1)
import glutil
########################################################################
#
# Global Constants
#
########################################################################

from const import version

weights_map = {
    "Lower"			: "l",
    "Boost"			: "b",
    "Shift Tuple Weights"	: "s"
    }

simplify_map = {
    "Add"		: "a",
    "Remove"		: "r"
    }

########################################################################
#
# Bayes Classifier App
#
########################################################################

class BayesClassifierApp(glutil.GnomeApp):

    def __init__(self, dom="", tab=""):
        glutil.GnomeApp.__init__(self,
                                 "gdmbayes",
                                 "Gnome Data Miner: Bayes Classifier",
                                 version,
                                 glutil.appdir("gdmbayes.glade"),
                                 "bayes_app")
        
        self.default_data_ext  = "tab"
        self.default_names_ext = "dom"

        #
        # Temporary Files
        #
        self.bc_file = tempfile.mktemp(".bc")
        self.dtp_file = tempfile.mktemp(".dtp")
        self.rs_file = tempfile.mktemp(".rs")
        self.log_file = tempfile.mktemp(".gdmbayes-log")

        if tab != "":
            self.dataset_file_entry.gtk_entry().set_text(tab)
            self.dataset_file_entry.gnome_entry().append_history(1, tab)

        if dom != "":
            self.description_file_entry.gtk_entry().set_text(dom)
            self.description_file_entry.gnome_entry().append_history(1, dom)

    #
    # Command line generation and execution
    #

    def get_dataset_options(self):
        command = ""
        #
        # Characters to be treated as blank
        #
        blanks = self.blank_entry.get_text()
        if blanks != "": command += "-b\"" + blanks + "\" "
        #
        # Use default header (column count)
        #
        if self.number_columns_checkbutton.get_active(): command += "-d "
        #
        # Dataset field separator
        #
        field_separator = self.field_separator_entry.get_text()
        if field_separator != "": command += "-f\"" + field_separator + "\" "
        #
        # Include a record count
        #
        if self.record_count_checkbutton.get_active(): command += "-n "
        #
        # Dataset record separator
        #
        record_separator = self.record_separator_entry.get_text()
        if record_separator != "": command += "-r\"" + record_separator + "\" "
        #
        # Unknown value character
        #
        unknown = self.unknown_entry.get_text()
        if unknown != "": command += "-u\"" + unknown + "\" "
        #
        # All done
        #
        return(command)
        

    def execute_bci(self):
        #
        # What do we generate: Naive or Full Bayes
        #
        generate_naive  = self.naive_checkbutton.get_active()
        #X#prune_tree = self.prune_checkbutton.get_active()
        #X#generate_rules = self.rules_checkbutton.get_active()
        #
        # Relevant files
        #
        data_file = self.dataset_file_entry.gtk_entry().get_text()
        desc_file = self.description_file_entry.gtk_entry().get_text()

        if data_file == "":
            glutil.WarningDialog(self,
                          "Please enter the name of a dataset file first.")
        elif desc_file == "":
            glutil.WarningDialog(self,
                          "Please enter the name of a description file first.")
        elif not os.path.exists(data_file):
            glutil.ErrorDialog(self,
                        "Dataset file could not be found: " + data_file)
        elif not os.path.exists(desc_file):
            glutil.ErrorDialog(self,
                        "Description file could not be found: " + desc_file)
        else:
            #
            # BUILD THE COMMAND
            #
            command = "bci "
            #
            # Add dataset options
            #
            command += self.get_dataset_options()
            #
            # Specify the target column.  If blank the the default
            # last column is used.
            #
            target_column = self.target_column_entry.get_text()
            if target_column != "": command += "-c" + target_column + " "
            #
            # Naive or Full Bayes?
            #
            if not self.naive_checkbutton.get_active(): command += "-F "
            #
            # Maximum Likelihood
            #
            if self.max_likelihood_checkbutton.get_active(): command += "-m "
            #
            # Maximum line length
            #
            command += "-l%d " % self.line_spinbutton.get_value()
            #
            # Distribute Weights
            #
            if self.distribute_weights_checkbutton.get_active():
                command += "-t "
            #
            # Output relative frequencies
            #
            if self.frequency_checkbutton.get_active(): command += "-p "
            #
            # Balance Class Frequencies
            #
            if self.balance_checkbutton.get_active():
                method = self.balance_combo_entry.get_text()
                command += "-w" + weights_map[method] + " "
            #
            # Simplification
            #
            if self.simplify_checkbutton.get_active():
                method = self.simplify_combo_entry.get_text()
                command += "-s" + simplify_map[method] + " "
            #
            # Laplace correction
            #
            laplace = self.laplace_spinbutton.get_value()
            if laplace > 0: command += "-l%d " % laplace
            #
            # Add file names to command line
            #
            command += desc_file + " " + data_file + " " + self.bc_file
            #
            # Trace of command being executed
            #
            glutil.start_log(self.log_file, command)
            #
            # Redirect output to a file
            #
            command += " 2>> " + self.log_file
            #
            # RUN THE COMMAND
            #
            # print command
            os.system(command)
            glutil.end_log(self.log_file)
            glutil.FileViewWindow(self.bc_file,
                                  "Bayes Classifier View",
                                  os.path.splitext(data_file)[0] + ".bc")

    def on_view_log_button_clicked(self, *extra):
        if not os.path.exists(self.log_file):
            glutil.WarningDialog(self,
                                "No log file yet.")
        else:
            glutil.FileViewWindow(self.log_file, "Log File", "", clearable=1)

    def cleanup(self):
        if os.path.isfile(self.bc_file): os.remove(self.bc_file)
        if os.path.isfile(self.log_file): os.remove(self.log_file)

    #
    # global
    #
    def on_help_about_activate(self, *extra):
        aboutxml = gtk.glade.XML(glutil.appdir("gdmbayes.glade"),"about")
        about = aboutxml.get_widget("about")
        about.set_property("name", "Gnome Data Miner: Bayes Classifier")
        about.set_property("version", version)
        about.show()
    def on_quit_activate(self, *extra):
        self.cleanup()
        self.quit()
    def on_quit_activate(self, *extra):
        self.cleanup()
        self.quit()

    #
    # implementation
    #

    def on_execute_button_clicked(self, *extra):
        self.execute_bci()

    def on_generate_description_button_clicked(self, *extra):
        tabfile = self.dataset_file_entry.gtk_entry().get_text()
        domfile = self.description_file_entry.gtk_entry().get_text()
        if tabfile == "":
            glutil.WarningDialog(self,
                          "Please specify the filename for " +\
                          "the dataset file first. Often they have " +\
                          "a .tab extension")
        elif not os.path.exists(tabfile):
            glutil.WarningDialog(self, "The specified data file does not exist. " +\
                          "Please supply an existing data file. Often " +\
                          "they have a .tab extension")
        elif domfile == "":
            glutil.WarningDialog(self, "Please specify the filename for " +\
                          "writing the description to. Often this " +\
                          "will have an extension of .dom (for domain).")
        elif not os.path.exists(domfile) \
             or glutil.QuestionDialog(self, "This will overwrite " + domfile +\
                               ". Do you wish to continue?").get_answer():
            command = "dom "
            #
            # Add dataset options.
            #
            command += self.get_dataset_options()
            #
            # Automatic type determination.
            #
            command += "-a "
            #
            # Add the data files.
            #
            command += tabfile + " "
            command += domfile + " "
            #
            # Trace of command being executed
            #
            glutil.start_log(self.log_file, command)
            command += " 2>> " + self.log_file
            os.system(command)
            glutil.end_log(self.log_file)
            glutil.FileViewWindow(domfile,
                           "Domain File",
                           domfile, editable=1)

    #
    # Buttons
    #

    def on_view_dataset_button_clicked(self, *extra):
        tabfile = self.dataset_file_entry.gtk_entry().get_text()
        if tabfile == "":
            glutil.WarningDialog(self,
                          "Please specify the filename for " +\
                          "the dataset file first. Often they have " +\
                          "a .tab extension")
        elif not os.path.exists(tabfile):
            glutil.WarningDialog(self, "The specified data file does not exist. " +\
                          "Please supply an existing data file. Often " +\
                          "they have a .tab extension")
        else:
            glutil.FileViewWindow(tabfile, "Dataset File", tabfile, saveable=0)

        
    def on_view_description_button_clicked(self, *extra):
        domfile = self.description_file_entry.gtk_entry().get_text()
        if domfile == "":
            glutil.WarningDialog(self,
                          "Please specify the filename for " +\
                          "the description file first. Often they have " +\
                          "a .dom extension")
        elif not os.path.exists(domfile):
            glutil.WarningDialog(self, "The specified description file does not exist. " +\
                          "Please supply an existing description file. Often " +\
                          "they have a .dom extension")
        else:
            glutil.FileViewWindow(domfile, "Description File", domfile, saveable=0)


    def on_new_button_clicked(self, *extra):
        self.reset_defaults()
        self.dataset_file_entry.gtk_entry().set_text("")
        self.description_file_entry.gtk_entry().set_text("")

    def on_reset_button_clicked(self, *extra):
        self.reset_defaults()

    def reset_defaults(self):
        self.line_spinbutton.set_value(70)
        
        self.naive_checkbutton.set_active(gtk.TRUE)
        self.max_likelihood_checkbutton.set_active(gtk.FALSE)
        self.distribute_weights_checkbutton.set_active(gtk.FALSE)
        self.frequency_checkbutton.set_active(gtk.FALSE)
        self.balance_checkbutton.set_active(gtk.FALSE)
        self.balance_combo.set_sensitive(gtk.FALSE)
        self.balance_combo_entry.set_text("Lower")
        self.simplify_checkbutton.set_active(gtk.FALSE)
        self.simplify_combo.set_sensitive(gtk.FALSE)
        self.simplify_combo_entry.set_text("Add")
        self.laplace_spinbutton.set_value(0)

        self.target_column_entry.set_text("")

    def on_save_button_clicked(self, *extra):
        #
        # This is a convenient place to try things out
        #
        return
        #glutil.FileViewWindow("drug.dom", "Drug Data File")
        dlg=gtk.MessageDialog(self, gtk.DIALOG_DESTROY_WITH_PARENT,
                              gtk.MESSAGE_WARNING,
                              gtk.BUTTONS_OK,
                              "Please specify a dataset file name before executing")
        
        dlg.run()
        dlg.destroy()
        #MessageDialog("Please supply the name of a data file")
        #self.statusbar.add_status("Hi THere")

    def on_dataset_file_entry_activate(self, *extra):
        dataset_entry =  self.dataset_file_entry.gtk_entry()
        description_entry =  self.description_file_entry.gtk_entry()
        filename = dataset_entry.get_text()
        description = description_entry.get_text()
        (path,extension) = os.path.splitext(filename)
        if extension == "" \
           and not os.path.exists(path) \
           and os.path.exists(path + ".tab"):
            dataset_entry.set_text(path + ".tab")
            description_entry.set_text(path + ".dom")

    #
    # Checkbox/Combo Box links
    #
    def on_balance_checkbutton_toggled(self, *extra):
        is_active = self.balance_checkbutton.get_active()
        self.balance_combo.set_sensitive(is_active)
    def on_simplify_checkbutton_toggled(self, *extra):
        is_active = self.simplify_checkbutton.get_active()
        self.simplify_combo.set_sensitive(is_active)

    def on_naive_checkbutton_toggled(self, *extra):
        is_active = self.naive_checkbutton.get_active()
        self.simplify_checkbutton.set_sensitive(is_active)

    def on_classification_radiobutton_toggled(self, *extra):
        is_classification = self.classification_radiobutton.get_active()
        is_regression = self.regression_radiobutton.get_active()
        if is_classification:
            self.set_measure_classification()
        elif is_regression:
            self.set_measure_regression()


    #
    # Rule Options
    #
    def on_rules_checkbutton_toggled(self, *extra):
        is_active = self.rules_checkbutton.get_active()
        self.rules_support_checkbutton.set_sensitive(is_active)
        self.rules_confidence_checkbutton.set_sensitive(is_active)
        self.rules_onecond_checkbutton.set_sensitive(is_active)

    #
    # Prune Options
    #
    def on_prune_checkbutton_toggled(self, *extra):
        is_active = self.prune_checkbutton.get_active()
        self.prune_method_combo.set_sensitive(is_active)
        self.prune_method_label.set_sensitive(is_active)
        self.prune_parameter_hscale.set_sensitive(is_active)
        self.prune_parameter_label.set_sensitive(is_active)
        self.prune_largest_checkbutton.set_sensitive(is_active)
        self.prune_data_checkbutton.set_sensitive(is_active)

    #
    # dummies
    #
    def on_not_implemented(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_save1_activate(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_preferences1_activate(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_cut1_activate(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_open1_activate(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_paste1_activate(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_clear1_activate(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_new1_activate(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_save_as1_activate(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_properties1_activate(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_copy1_activate(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_new_button_clicked(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_open_button_clicked(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")
    def on_save_button_clicked(self, *extra):
        glutil.WarningDialog(self, "This function is not yet implemented.")

########################################################################
#
# AppStatusBar : Try figure it out
#
########################################################################

class AppStatusBar:

    def __init__(self, appbar):
        self.appbar = appbar
        self.statusmessages = []
        self.statuscount = []

    def add_status(self, status, timeout=4000, allow_duplicate=0):
        if not allow_duplicate:
            try:
                dup = self.statusmessages.index(status)
            except ValueError:
                pass
            else:
                self.statuscount[dup] += 1
                gtk.timeout_add(timeout, lambda x: self._remove_status(status), 0)
                return

        self.statusmessages.append(status)
        message = self._get_status_message()
        if len(self.statusmessages)==1:
            self.appbar.push(message)
        else:
            self.appbar.set_status(message)
        gtk.timeout_add(timeout, lambda x: self._remove_status(status), 0)
        self.statuscount.append(1)

    def _get_status_message(self):
        return "[%s]" % "] [".join(self.statusmessages)

    def _remove_status(self, status):
        i = self.statusmessages.index(status)
        if self.statuscount[i] == 1:
            self.statusmessages.pop(i)
            self.statuscount.pop(i)
            if len(self.statusmessages)==0:
                self.appbar.pop()
            else:
                message = self._get_status_message()
                self.appbar.set_status(message)
        else:
            self.statuscount[i] -= 1

########################################################################
#
# Main
#
########################################################################

def main():
    if len(sys.argv) == 1:
        BayesClassifierApp().mainloop()
    elif len(sys.argv) == 2:
        BayesClassifierApp(sys.argv[1] + ".dom", sys.argv[1] + ".tab").mainloop()

if __name__ == "__main__":
    main()
